@@ -219,7 +219,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
      * @param name pathname
      * @param simpleName actual filename.
      */
-    public DumpArchiveEntry(final String name, final String simpleName) {
+    public DumpArchiveEntry(String name, String simpleName) {
         setName(name);
         this.simpleName = simpleName;
     }
@@ -227,13 +227,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
     /**
      * Constructor taking name, inode and type.
      *
-     * @param name the name
-     * @param simpleName the simple name
-     * @param ino the ino
-     * @param type the type
+     * @param name
+     * @param simpleName
+     * @param ino
+     * @param type
      */
-    protected DumpArchiveEntry(final String name, final String simpleName, final int ino,
-                               final TYPE type) {
+    protected DumpArchiveEntry(String name, String simpleName, int ino,
+                               TYPE type) {
         setType(type);
         setName(name);
         this.simpleName = simpleName;
@@ -241,6 +241,12 @@ public class DumpArchiveEntry implements ArchiveEntry {
         this.offset = 0;
     }
 
+    /**
+     * Constructor taking tape buffer.
+     * @param buffer
+     * @param offset
+     */
+
     /**
      * Returns the path of the entry.
      * @return the path of the entry.
@@ -251,15 +257,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Sets the path of the entry.
-     * @param simpleName the simple name
      */
-    protected void setSimpleName(final String simpleName) {
+    protected void setSimpleName(String simpleName) {
         this.simpleName = simpleName;
     }
 
     /**
      * Returns the ino of the entry.
-     * @return the ino
      */
     public int getIno() {
         return header.getIno();
@@ -267,7 +271,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Return the number of hard links to the entry.
-     * @return the number of hard links
      */
     public int getNlink() {
         return nlink;
@@ -275,15 +278,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the number of hard links.
-     * @param nlink the number of hard links
      */
-    public void setNlink(final int nlink) {
+    public void setNlink(int nlink) {
         this.nlink = nlink;
     }
 
     /**
      * Get file creation time.
-     * @return the creation time
      */
     public Date getCreationTime() {
         return new Date(ctime);
@@ -291,15 +292,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the file creation time.
-     * @param ctime the creation time
      */
-    public void setCreationTime(final Date ctime) {
+    public void setCreationTime(Date ctime) {
         this.ctime = ctime.getTime();
     }
 
     /**
      * Return the generation of the file.
-     * @return the generation
      */
     public int getGeneration() {
         return generation;
@@ -307,15 +306,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the generation of the file.
-     * @param generation the generation
      */
-    public void setGeneration(final int generation) {
+    public void setGeneration(int generation) {
         this.generation = generation;
     }
 
     /**
      * Has this file been deleted? (On valid on incremental dumps.)
-     * @return whether the file has been deleted
      */
     public boolean isDeleted() {
         return isDeleted;
@@ -323,15 +320,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set whether this file has been deleted.
-     * @param isDeleted whether the file has been deleted
      */
-    public void setDeleted(final boolean isDeleted) {
+    public void setDeleted(boolean isDeleted) {
         this.isDeleted = isDeleted;
     }
 
     /**
      * Return the offset within the archive
-     * @return the offset
      */
     public long getOffset() {
         return offset;
@@ -339,15 +334,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the offset within the archive.
-     * @param offset the offset
      */
-    public void setOffset(final long offset) {
+    public void setOffset(long offset) {
         this.offset = offset;
     }
 
     /**
      * Return the tape volume where this file is located.
-     * @return the volume
      */
     public int getVolume() {
         return volume;
@@ -355,15 +348,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the tape volume.
-     * @param volume the volume
      */
-    public void setVolume(final int volume) {
+    public void setVolume(int volume) {
         this.volume = volume;
     }
 
     /**
      * Return the type of the tape segment header.
-     * @return the segment header
      */
     public DumpArchiveConstants.SEGMENT_TYPE getHeaderType() {
         return header.getType();
@@ -371,7 +362,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Return the number of records in this segment.
-     * @return the number of records
      */
     public int getHeaderCount() {
         return header.getCount();
@@ -379,7 +369,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Return the number of sparse records in this segment.
-     * @return the number of sparse records
      */
     public int getHeaderHoles() {
         return header.getHoles();
@@ -387,27 +376,31 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a sparse record?
-     * @param idx index of the record to check
-     * @return whether this is a sparse record
      */
-    public boolean isSparseRecord(final int idx) {
+    public boolean isSparseRecord(int idx) {
         return (header.getCdata(idx) & 0x01) == 0;
     }
 
+    /**
+     * @see java.lang.Object#hashCode()
+     */
     @Override
     public int hashCode() {
         return ino;
     }
 
+    /**
+     * @see java.lang.Object#equals(Object o)
+     */
     @Override
-    public boolean equals(final Object o) {
+    public boolean equals(Object o) {
         if (o == this) {
             return true;
         } else if (o == null || !o.getClass().equals(getClass())) {
             return false;
         }
 
-        final DumpArchiveEntry rhs = (DumpArchiveEntry) o;
+        DumpArchiveEntry rhs = (DumpArchiveEntry) o;
 
         if ((header == null) || (rhs.header == null)) {
             return false;
@@ -425,6 +418,9 @@ public class DumpArchiveEntry implements ArchiveEntry {
         return true;
     }
 
+    /**
+     * @see java.lang.Object#toString()
+     */
     @Override
     public String toString() {
         return getName();
@@ -434,11 +430,12 @@ public class DumpArchiveEntry implements ArchiveEntry {
      * Populate the dump archive entry and tape segment header with
      * the contents of the buffer.
      *
-     * @param buffer buffer to read content from
+     * @param buffer
+     * @throws Exception
      */
-    static DumpArchiveEntry parse(final byte[] buffer) {
-        final DumpArchiveEntry entry = new DumpArchiveEntry();
-        final TapeSegmentHeader header = entry.header;
+    static DumpArchiveEntry parse(byte[] buffer) {
+        DumpArchiveEntry entry = new DumpArchiveEntry();
+        TapeSegmentHeader header = entry.header;
 
         header.type = DumpArchiveConstants.SEGMENT_TYPE.find(DumpArchiveUtil.convert32(
                     buffer, 0));
@@ -452,7 +449,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
         //header.magic = DumpArchiveUtil.convert32(buffer, 24);
         //header.checksum = DumpArchiveUtil.convert32(buffer, 28);
-        final int m = DumpArchiveUtil.convert16(buffer, 32);
+        int m = DumpArchiveUtil.convert16(buffer, 32);
 
         // determine the type of the file.
         entry.setType(TYPE.find((m >> 12) & 0x0F));
@@ -503,7 +500,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
     /**
      * Update entry with information from next tape segment header.
      */
-    void update(final byte[] buffer) {
+    void update(byte[] buffer) {
         header.volume = DumpArchiveUtil.convert32(buffer, 16);
         header.count = DumpArchiveUtil.convert32(buffer, 160);
 
@@ -542,7 +539,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
             return ino;
         }
 
-        void setIno(final int ino) {
+        void setIno(int ino) {
             this.ino = ino;
         }
 
@@ -554,7 +551,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
             return holes;
         }
 
-        public int getCdata(final int idx) {
+        public int getCdata(int idx) {
             return cdata[idx];
         }
     }
@@ -563,7 +560,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
      * Returns the name of the entry.
      * @return the name of the entry.
      */
-    @Override
     public String getName() {
         return name;
     }
@@ -578,7 +574,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Sets the name of the entry.
-     * @param name the name
      */
     public final void setName(String name) {
         this.originalName = name;
@@ -593,27 +588,19 @@ public class DumpArchiveEntry implements ArchiveEntry {
         this.name = name;
     }
 
-    /**
-     * The last modified date.
-     * @return the last modified date
-     */
-    @Override
     public Date getLastModifiedDate() {
         return new Date(mtime);
     }
 
     /**
      * Is this a directory?
-     * @return whether this is a directory
      */
-    @Override
     public boolean isDirectory() {
         return type == TYPE.DIRECTORY;
     }
 
     /**
      * Is this a regular file?
-     * @return whether this is a regular file
      */
     public boolean isFile() {
         return type == TYPE.FILE;
@@ -621,7 +608,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a network device?
-     * @return whether this is a socket
      */
     public boolean isSocket() {
         return type == TYPE.SOCKET;
@@ -629,7 +615,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a character device?
-     * @return whether this is a character device
      */
     public boolean isChrDev() {
         return type == TYPE.CHRDEV;
@@ -637,7 +622,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a block device?
-     * @return whether this is a block device
      */
     public boolean isBlkDev() {
         return type == TYPE.BLKDEV;
@@ -645,7 +629,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Is this a fifo/pipe?
-     * @return whether this is a fifo
      */
     public boolean isFifo() {
         return type == TYPE.FIFO;
@@ -653,7 +636,6 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Get the type of the entry.
-     * @return the type
      */
     public TYPE getType() {
         return type;
@@ -661,15 +643,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the type of the entry.
-     * @param type the type
      */
-    public void setType(final TYPE type) {
+    public void setType(TYPE type) {
         this.type = type;
     }
 
     /**
      * Return the access permissions on the entry.
-     * @return the access permissions
      */
     public int getMode() {
         return mode;
@@ -677,16 +657,14 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the access permissions on the entry.
-     * @param mode the access permissions
      */
-    public void setMode(final int mode) {
+    public void setMode(int mode) {
         this.mode = mode & 07777;
         this.permissions = PERMISSION.find(mode);
     }
 
     /**
      * Returns the permissions on the entry.
-     * @return the permissions
      */
     public Set<PERMISSION> getPermissions() {
         return permissions;
@@ -694,9 +672,7 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Returns the size of the entry.
-     * @return the size
      */
-    @Override
     public long getSize() {
         return isDirectory() ? SIZE_UNKNOWN : size;
     }
@@ -710,23 +686,20 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the size of the entry.
-     * @param size the size
      */
-    public void setSize(final long size) {
+    public void setSize(long size) {
         this.size = size;
     }
 
     /**
      * Set the time the file was last modified.
-     * @param mtime the last modified time
      */
-    public void setLastModifiedDate(final Date mtime) {
+    public void setLastModifiedDate(Date mtime) {
         this.mtime = mtime.getTime();
     }
 
     /**
      * Returns the time the file was last accessed.
-     * @return the access time
      */
     public Date getAccessTime() {
         return new Date(atime);
@@ -734,15 +707,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the time the file was last accessed.
-     * @param atime the access time
      */
-    public void setAccessTime(final Date atime) {
+    public void setAccessTime(Date atime) {
         this.atime = atime.getTime();
     }
 
     /**
      * Return the user id.
-     * @return the user id
      */
     public int getUserId() {
         return uid;
@@ -750,15 +721,13 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the user id.
-     * @param uid the user id
      */
-    public void setUserId(final int uid) {
+    public void setUserId(int uid) {
         this.uid = uid;
     }
 
     /**
      * Return the group id
-     * @return the group id
      */
     public int getGroupId() {
         return gid;
@@ -766,9 +735,8 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
     /**
      * Set the group id.
-     * @param gid the group id
      */
-    public void setGroupId(final int gid) {
+    public void setGroupId(int gid) {
         this.gid = gid;
     }
 
@@ -785,14 +753,14 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
         private int code;
 
-        private TYPE(final int code) {
+        private TYPE(int code) {
             this.code = code;
         }
 
-        public static TYPE find(final int code) {
+        public static TYPE find(int code) {
             TYPE type = UNKNOWN;
 
-            for (final TYPE t : TYPE.values()) {
+            for (TYPE t : TYPE.values()) {
                 if (code == t.code) {
                     type = t;
                 }
@@ -818,14 +786,14 @@ public class DumpArchiveEntry implements ArchiveEntry {
 
         private int code;
 
-        private PERMISSION(final int code) {
+        private PERMISSION(int code) {
             this.code = code;
         }
 
-        public static Set<PERMISSION> find(final int code) {
-            final Set<PERMISSION> set = new HashSet<>();
+        public static Set<PERMISSION> find(int code) {
+            Set<PERMISSION> set = new HashSet<PERMISSION>();
 
-            for (final PERMISSION p : PERMISSION.values()) {
+            for (PERMISSION p : PERMISSION.values()) {
                 if ((code & p.code) == p.code) {
                     set.add(p);
                 }
