@@ -126,7 +126,7 @@ public class SevenZFile implements Closeable {
      * @since 1.19
      */
     public SevenZFile(final File fileName, final char[] password, SevenZFileOptions options) throws IOException {
-        this(Files.newByteChannel(fileName.toPath(), EnumSet.of(StandardOpenOption.READ)),
+        this(Files.newByteChannel(fileName.toPath(), EnumSet.of(StandardOpenOption.READ)), // NOSONAR
                 fileName.getAbsolutePath(), utf16Decode(password), true, options);
     }
 
@@ -140,6 +140,7 @@ public class SevenZFile implements Closeable {
      * @throws IOException if reading the archive fails
      * @deprecated use the char[]-arg version for the password instead
      */
+    @Deprecated
     public SevenZFile(final File fileName, final byte[] password) throws IOException {
         this(Files.newByteChannel(fileName.toPath(), EnumSet.of(StandardOpenOption.READ)),
                 fileName.getAbsolutePath(), password, true, SevenZFileOptions.DEFAULT);
@@ -298,6 +299,7 @@ public class SevenZFile implements Closeable {
      * @since 1.13
      * @deprecated use the char[]-arg version for the password instead
      */
+    @Deprecated
     public SevenZFile(final SeekableByteChannel channel,
                       final byte[] password) throws IOException {
         this(channel, DEFAULT_FILE_NAME, password);
@@ -319,6 +321,7 @@ public class SevenZFile implements Closeable {
      * @since 1.13
      * @deprecated use the char[]-arg version for the password instead
      */
+    @Deprecated
     public SevenZFile(final SeekableByteChannel channel, String fileName,
                       final byte[] password) throws IOException {
         this(channel, fileName, password, false, SevenZFileOptions.DEFAULT);
@@ -402,7 +405,7 @@ public class SevenZFile implements Closeable {
         if (entry.getName() == null && options.getUseDefaultNameForUnnamedEntries()) {
             entry.setName(getDefaultName());
         }
-        buildDecodingStream();
+        buildDecodingStream(currentEntryIndex);
         uncompressedBytesReadFromCurrentEntry = compressedBytesReadFromCurrentEntry = 0;
         return entry;
     }
@@ -441,17 +444,86 @@ public class SevenZFile implements Closeable {
                     archiveVersionMajor, archiveVersionMinor));
         }
 
+        boolean headerLooksValid = false;  // See https://www.7-zip.org/recover.html - "There is no correct End Header at the end of archive"
         final long startHeaderCrc = 0xffffFFFFL & buf.getInt();
-        final StartHeader startHeader = readStartHeader(startHeaderCrc);
+        if (startHeaderCrc == 0) {
+            // This is an indication of a corrupt header - peek the next 20 bytes
+            long currentPosition = channel.position();
+            ByteBuffer peekBuf = ByteBuffer.allocate(20);
+            readFully(peekBuf);
+            channel.position(currentPosition);
+            // Header invalid if all data is 0
+            while (peekBuf.hasRemaining()) {
+                if (peekBuf.get()!=0) {
+                    headerLooksValid = true;
+                    break;
+                }
+            }
+        } else {
+            headerLooksValid = true;
+        }
+
+        if (headerLooksValid) {
+            final StartHeader startHeader = readStartHeader(startHeaderCrc);
+            return initializeArchive(startHeader, password, true);
+        } else {
+            // No valid header found - probably first file of multipart archive was removed too early. Scan for end header.
+            return tryToLocateEndHeader(password);
+        }
+    }
+
+    private Archive tryToLocateEndHeader(final byte[] password) throws IOException {
+        ByteBuffer nidBuf = ByteBuffer.allocate(1);
+        final long searchLimit = 1024l * 1024 * 1;
+        // Main header, plus bytes that readStartHeader would read
+        final long previousDataSize = channel.position() + 20;
+        final long minPos;
+        // Determine minimal position - can't start before current position
+        if (channel.position() + searchLimit > channel.size()) {
+            minPos = channel.position();
+        } else {
+            minPos = channel.size() - searchLimit;
+        }
+        long pos = channel.size() - 1;
+        // Loop: Try from end of archive
+        while (pos > minPos) {
+            pos--;
+            channel.position(pos);
+            nidBuf.rewind();
+            channel.read(nidBuf);
+            int nid = nidBuf.array()[0];
+            // First indicator: Byte equals one of these header identifiers
+            if (nid == NID.kEncodedHeader || nid == NID.kHeader) {
+                try {
+                    // Try to initialize Archive structure from here
+                    final StartHeader startHeader = new StartHeader();
+                    startHeader.nextHeaderOffset = pos - previousDataSize;
+                    startHeader.nextHeaderSize = channel.size() - pos;
+                    Archive result = initializeArchive(startHeader, password, false);
+                    // Sanity check: There must be some data...
+                    if (result.packSizes != null && result.files.length > 0) {
+                        return result;
+                    }
+                } catch (Exception ignore) {
+                    // Wrong guess...
+                }
+            }
+        }
+        throw new IOException("Start header corrupt and unable to guess end header");
+    }
+
+    private Archive initializeArchive(StartHeader startHeader, final byte[] password, boolean verifyCrc) throws IOException {
         assertFitsIntoInt("nextHeaderSize", startHeader.nextHeaderSize);
         final int nextHeaderSizeInt = (int) startHeader.nextHeaderSize;
         channel.position(SIGNATURE_HEADER_SIZE + startHeader.nextHeaderOffset);
-        buf = ByteBuffer.allocate(nextHeaderSizeInt).order(ByteOrder.LITTLE_ENDIAN);
+        ByteBuffer buf = ByteBuffer.allocate(nextHeaderSizeInt).order(ByteOrder.LITTLE_ENDIAN);
         readFully(buf);
-        final CRC32 crc = new CRC32();
-        crc.update(buf.array());
-        if (startHeader.nextHeaderCrc != crc.getValue()) {
-            throw new IOException("NextHeader CRC mismatch");
+        if (verifyCrc) {
+            final CRC32 crc = new CRC32();
+            crc.update(buf.array());
+            if (startHeader.nextHeaderCrc != crc.getValue()) {
+                throw new IOException("NextHeader CRC mismatch");
+            }
         }
 
         Archive archive = new Archive();
@@ -781,8 +853,8 @@ public class SevenZFile implements Closeable {
             }
             // would need to keep looping as above:
             while (moreAlternativeMethods) {
-                throw new IOException("Alternative methods are unsupported, please report. " +
-                        "The reference implementation doesn't support them either.");
+                throw new IOException("Alternative methods are unsupported, please report. " + // NOSONAR
+                    "The reference implementation doesn't support them either.");
             }
         }
         folder.coders = coders;
@@ -1002,6 +1074,9 @@ public class SevenZFile implements Closeable {
         for (int i = 0; i < files.length; i++) {
             files[i].setHasStream(isEmptyStream == null || !isEmptyStream.get(i));
             if (files[i].hasStream()) {
+                if (archive.subStreamsInfo == null) {
+                    throw new IOException("Archive contains file with streams but no subStreamsInfo");
+                }
                 files[i].setDirectory(false);
                 files[i].setAntiItem(false);
                 files[i].setHasCrc(archive.subStreamsInfo.hasCrc.get(nonEmptyFileCounter));
@@ -1073,15 +1148,23 @@ public class SevenZFile implements Closeable {
         archive.streamMap = streamMap;
     }
 
-    private void buildDecodingStream() throws IOException {
-        final int folderIndex = archive.streamMap.fileFolderIndex[currentEntryIndex];
+    private void buildDecodingStream(int entryIndex) throws IOException {
+        if (archive.streamMap == null) {
+            throw new IOException("Archive doesn't contain stream information to read entries");
+        }
+        final int folderIndex = archive.streamMap.fileFolderIndex[entryIndex];
         if (folderIndex < 0) {
             deferredBlockStreams.clear();
             // TODO: previously it'd return an empty stream?
             // new BoundedInputStream(new ByteArrayInputStream(new byte[0]), 0);
             return;
         }
-        final SevenZArchiveEntry file = archive.files[currentEntryIndex];
+        final SevenZArchiveEntry file = archive.files[entryIndex];
+        boolean isInSameFolder = false;
+        final Folder folder = archive.folders[folderIndex];
+        final int firstPackStreamIndex = archive.streamMap.folderFirstPackStreamIndex[folderIndex];
+        final long folderOffset = SIGNATURE_HEADER_SIZE + archive.packPos +
+                archive.streamMap.packStreamOffsets[firstPackStreamIndex];
         if (currentFolderIndex == folderIndex) {
             // (COMPRESS-320).
             // The current entry is within the same (potentially opened) folder. The
@@ -1089,7 +1172,17 @@ public class SevenZFile implements Closeable {
             // but don't do it eagerly -- if the user skips over the entire folder nothing
             // is effectively decompressed.
 
-            file.setContentMethods(archive.files[currentEntryIndex - 1].getContentMethods());
+            file.setContentMethods(archive.files[entryIndex - 1].getContentMethods());
+
+            // if this is called in a random access, then the content methods of previous entry may be null
+            // the content methods should be set to methods of the first entry as it must not be null,
+            // and the content methods would only be set if the content methods was not set
+            if(currentEntryIndex != entryIndex && file.getContentMethods() == null) {
+                int folderFirstFileIndex = archive.streamMap.folderFirstFileIndex[folderIndex];
+                SevenZArchiveEntry folderFirstFile = archive.files[folderFirstFileIndex];
+                file.setContentMethods(folderFirstFile.getContentMethods());
+            }
+            isInSameFolder = true;
         } else {
             // We're opening a new folder. Discard any queued streams/ folder stream.
             currentFolderIndex = folderIndex;
@@ -1099,13 +1192,41 @@ public class SevenZFile implements Closeable {
                 currentFolderInputStream = null;
             }
 
-            final Folder folder = archive.folders[folderIndex];
-            final int firstPackStreamIndex = archive.streamMap.folderFirstPackStreamIndex[folderIndex];
-            final long folderOffset = SIGNATURE_HEADER_SIZE + archive.packPos +
-                    archive.streamMap.packStreamOffsets[firstPackStreamIndex];
             currentFolderInputStream = buildDecoderStack(folder, folderOffset, firstPackStreamIndex, file);
         }
 
+        // if this mothod is called in a random access, then some entries need to be skipped,
+        // if the entry of entryIndex is in the same folder of the currentFolderIndex,
+        // then the entries between (currentEntryIndex + 1) and (entryIndex - 1) need to be skipped,
+        // otherwise it's a new folder, and the entries between firstFileInFolderIndex and (entryIndex - 1) need to be skipped
+        if(currentEntryIndex != entryIndex) {
+            int filesToSkipStartIndex = archive.streamMap.folderFirstFileIndex[folderIndex];
+            if(isInSameFolder) {
+                if(currentEntryIndex < entryIndex) {
+                    // the entries between filesToSkipStartIndex and currentEntryIndex had already been skipped
+                    filesToSkipStartIndex = currentEntryIndex + 1;
+                } else {
+                    // the entry is in the same folder of current entry, but it has already been read before, we need to reset
+                    // the position of the currentFolderInputStream to the beginning of folder, and then skip the files
+                    // from the start entry of the folder again
+                    deferredBlockStreams.clear();
+                    channel.position(folderOffset);
+                }
+            }
+
+            for(int i = filesToSkipStartIndex;i < entryIndex;i++) {
+                SevenZArchiveEntry fileToSkip = archive.files[i];
+                InputStream fileStreamToSkip = new BoundedInputStream(currentFolderInputStream, fileToSkip.getSize());
+                if (fileToSkip.getHasCrc()) {
+                    fileStreamToSkip = new CRC32VerifyingInputStream(fileStreamToSkip, fileToSkip.getSize(), fileToSkip.getCrcValue());
+                }
+                deferredBlockStreams.add(fileStreamToSkip);
+
+                // set the content methods as well, it equals to file.getContentMethods() because they are in same folder
+                fileToSkip.setContentMethods(file.getContentMethods());
+            }
+        }
+
         InputStream fileStream = new BoundedInputStream(currentFolderInputStream, file.getSize());
         if (file.getHasCrc()) {
             fileStream = new CRC32VerifyingInputStream(fileStream, file.getSize(), file.getCrcValue());
@@ -1134,6 +1255,9 @@ public class SevenZFile implements Closeable {
             }
             @Override
             public int read(final byte[] b, final int off, final int len) throws IOException {
+                if (len == 0) {
+                    return 0;
+                }
                 final int r = in.read(b, off, len);
                 if (r >= 0) {
                     count(r);
@@ -1199,6 +1323,37 @@ public class SevenZFile implements Closeable {
         return deferredBlockStreams.get(0);
     }
 
+    /**
+     * Returns an InputStream for reading the contents of the given entry.
+     *
+     * <p>For archives using solid compression randomly accessing
+     * entries will be significantly slower than reading the archive
+     * sequentiallly.</p>
+     *
+     * @param entry the entry to get the stream for.
+     * @return a stream to read the entry from.
+     * @throws IOException if unable to create an input stream from the zipentry
+     * @since Compress 1.20
+     */
+    public InputStream getInputStream(SevenZArchiveEntry entry) throws IOException {
+        int entryIndex = -1;
+        for (int i = 0; i < this.archive.files.length;i++) {
+            if (entry == this.archive.files[i]) {
+                entryIndex = i;
+                break;
+            }
+        }
+
+        if (entryIndex < 0) {
+            throw new IllegalArgumentException("Can not find " + entry.getName() + " in " + this.fileName);
+        }
+
+        buildDecodingStream(entryIndex);
+        currentEntryIndex = entryIndex;
+        currentFolderIndex = archive.streamMap.fileFolderIndex[entryIndex];
+        return getCurrentStream();
+    }
+
     /**
      * Reads data into an array of bytes.
      *
@@ -1222,6 +1377,9 @@ public class SevenZFile implements Closeable {
      *             if an I/O error has occurred
      */
     public int read(final byte[] b, final int off, final int len) throws IOException {
+        if (len == 0) {
+            return 0;
+        }
         int cnt = getCurrentStream().read(b, off, len);
         if (cnt > 0) {
             uncompressedBytesReadFromCurrentEntry += cnt;
