@@ -24,20 +24,11 @@ import java.io.DataOutputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
-import java.nio.ByteBuffer;
-import java.nio.ByteOrder;
-import java.nio.channels.SeekableByteChannel;
-import java.nio.file.Files;
-import java.nio.file.StandardOpenOption;
+import java.io.RandomAccessFile;
 import java.util.ArrayList;
 import java.util.BitSet;
-import java.util.Collections;
 import java.util.Date;
-import java.util.EnumSet;
-import java.util.HashMap;
 import java.util.List;
-import java.util.LinkedList;
-import java.util.Map;
 import java.util.zip.CRC32;
 
 import org.apache.commons.compress.archivers.ArchiveEntry;
@@ -48,102 +39,59 @@ import org.apache.commons.compress.utils.CountingOutputStream;
  * @since 1.6
  */
 public class SevenZOutputFile implements Closeable {
-    private final SeekableByteChannel channel;
-    private final List<SevenZArchiveEntry> files = new ArrayList<>();
+    private final RandomAccessFile file;
+    private final List<SevenZArchiveEntry> files = new ArrayList<SevenZArchiveEntry>();
     private int numNonEmptyStreams = 0;
-    private final CRC32 crc32 = new CRC32();
-    private final CRC32 compressedCrc32 = new CRC32();
+    private CRC32 crc32 = new CRC32();
+    private CRC32 compressedCrc32 = new CRC32();
     private long fileBytesWritten = 0;
     private boolean finished = false;
     private CountingOutputStream currentOutputStream;
-    private CountingOutputStream[] additionalCountingStreams;
-    private Iterable<? extends SevenZMethodConfiguration> contentMethods =
-            Collections.singletonList(new SevenZMethodConfiguration(SevenZMethod.LZMA2));
-    private final Map<SevenZArchiveEntry, long[]> additionalSizes = new HashMap<>();
-
+    private SevenZMethod contentCompression = SevenZMethod.LZMA2;
+    
     /**
      * Opens file to write a 7z archive to.
      *
-     * @param filename the file to write to
+     * @param filename name of the file to write to
      * @throws IOException if opening the file fails
      */
     public SevenZOutputFile(final File filename) throws IOException {
-        this(Files.newByteChannel(filename.toPath(),
-            EnumSet.of(StandardOpenOption.CREATE, StandardOpenOption.WRITE,
-                       StandardOpenOption.TRUNCATE_EXISTING)));
+        file = new RandomAccessFile(filename, "rw");
+        file.seek(SevenZFile.SIGNATURE_HEADER_SIZE);
     }
-
+    
     /**
-     * Prepares channel to write a 7z archive to.
-     *
-     * <p>{@link
-     * org.apache.commons.compress.utils.SeekableInMemoryByteChannel}
-     * allows you to write to an in-memory archive.</p>
-     *
-     * @param channel the channel to write to
-     * @throws IOException if the channel cannot be positioned properly
-     * @since 1.13
-     */
-    public SevenZOutputFile(final SeekableByteChannel channel) throws IOException {
-        this.channel = channel;
-        channel.position(SevenZFile.SIGNATURE_HEADER_SIZE);
-    }
-
-    /**
-     * Sets the default compression method to use for entry contents - the
+     * Sets the compression method to use for entry contents - the
      * default is LZMA2.
      *
      * <p>Currently only {@link SevenZMethod#COPY}, {@link
      * SevenZMethod#LZMA2}, {@link SevenZMethod#BZIP2} and {@link
      * SevenZMethod#DEFLATE} are supported.</p>
-     *
-     * <p>This is a short form for passing a single-element iterable
-     * to {@link #setContentMethods}.</p>
-     * @param method the default compression method
      */
-    public void setContentCompression(final SevenZMethod method) {
-        setContentMethods(Collections.singletonList(new SevenZMethodConfiguration(method)));
-    }
-
-    /**
-     * Sets the default (compression) methods to use for entry contents - the
-     * default is LZMA2.
-     *
-     * <p>Currently only {@link SevenZMethod#COPY}, {@link
-     * SevenZMethod#LZMA2}, {@link SevenZMethod#BZIP2} and {@link
-     * SevenZMethod#DEFLATE} are supported.</p>
-     *
-     * <p>The methods will be consulted in iteration order to create
-     * the final output.</p>
-     *
-     * @since 1.8
-     * @param methods the default (compression) methods
-     */
-    public void setContentMethods(final Iterable<? extends SevenZMethodConfiguration> methods) {
-        this.contentMethods = reverse(methods);
+    public void setContentCompression(SevenZMethod method) {
+        this.contentCompression = method;
     }
 
     /**
      * Closes the archive, calling {@link #finish} if necessary.
      * 
-     * @throws IOException on error
+     * @throws IOException
      */
-    @Override
     public void close() throws IOException {
         if (!finished) {
             finish();
         }
-        channel.close();
+        file.close();
     }
-
+    
     /**
      * Create an archive entry using the inputFile and entryName provided.
      * 
-     * @param inputFile file to create an entry from
-     * @param entryName the name to use
+     * @param inputFile
+     * @param entryName 
      * @return the ArchiveEntry set up with details from the file
      * 
-     * @throws IOException on error
+     * @throws IOException
      */
     public SevenZArchiveEntry createArchiveEntry(final File inputFile,
             final String entryName) throws IOException {
@@ -161,16 +109,16 @@ public class SevenZOutputFile implements Closeable {
      * {@link #closeArchiveEntry()} to complete the process.
      * 
      * @param archiveEntry describes the entry
-     * @throws IOException on error
+     * @throws IOException
      */
     public void putArchiveEntry(final ArchiveEntry archiveEntry) throws IOException {
         final SevenZArchiveEntry entry = (SevenZArchiveEntry) archiveEntry;
         files.add(entry);
     }
-
+    
     /**
      * Closes the archive entry.
-     * @throws IOException on error
+     * @throws IOException
      */
     public void closeArchiveEntry() throws IOException {
         if (currentOutputStream != null) {
@@ -179,21 +127,14 @@ public class SevenZOutputFile implements Closeable {
         }
 
         final SevenZArchiveEntry entry = files.get(files.size() - 1);
-        if (fileBytesWritten > 0) { // this implies currentOutputStream != null
+        if (fileBytesWritten > 0) {
             entry.setHasStream(true);
             ++numNonEmptyStreams;
-            entry.setSize(currentOutputStream.getBytesWritten()); //NOSONAR
+            entry.setSize(currentOutputStream.getBytesWritten());
             entry.setCompressedSize(fileBytesWritten);
             entry.setCrcValue(crc32.getValue());
             entry.setCompressedCrcValue(compressedCrc32.getValue());
             entry.setHasCrc(true);
-            if (additionalCountingStreams != null) {
-                final long[] sizes = new long[additionalCountingStreams.length];
-                for (int i = 0; i < additionalCountingStreams.length; i++) {
-                    sizes[i] = additionalCountingStreams[i].getBytesWritten();
-                }
-                additionalSizes.put(entry, sizes);
-            }
         } else {
             entry.setHasStream(false);
             entry.setSize(0);
@@ -201,12 +142,11 @@ public class SevenZOutputFile implements Closeable {
             entry.setHasCrc(false);
         }
         currentOutputStream = null;
-        additionalCountingStreams = null;
         crc32.reset();
         compressedCrc32.reset();
         fileBytesWritten = 0;
     }
-
+    
     /**
      * Writes a byte to the current archive entry.
      * @param b The byte to be written.
@@ -215,7 +155,7 @@ public class SevenZOutputFile implements Closeable {
     public void write(final int b) throws IOException {
         getCurrentOutputStream().write(b);
     }
-
+    
     /**
      * Writes a byte array to the current archive entry.
      * @param b The byte array to be written.
@@ -224,7 +164,7 @@ public class SevenZOutputFile implements Closeable {
     public void write(final byte[] b) throws IOException {
         write(b, 0, b.length);
     }
-
+    
     /**
      * Writes part of a byte array to the current archive entry.
      * @param b The byte array to be written.
@@ -237,7 +177,7 @@ public class SevenZOutputFile implements Closeable {
             getCurrentOutputStream().write(b, off, len);
         }
     }
-
+    
     /**
      * Finishes the addition of entries to this archive, without closing it.
      * 
@@ -248,47 +188,42 @@ public class SevenZOutputFile implements Closeable {
             throw new IOException("This archive has already been finished");
         }
         finished = true;
-
-        final long headerPosition = channel.position();
-
+        
+        final long headerPosition = file.getFilePointer();
+        
         final ByteArrayOutputStream headerBaos = new ByteArrayOutputStream();
         final DataOutputStream header = new DataOutputStream(headerBaos);
-
+        
         writeHeader(header);
         header.flush();
         final byte[] headerBytes = headerBaos.toByteArray();
-        channel.write(ByteBuffer.wrap(headerBytes));
-
+        file.write(headerBytes);
+        
         final CRC32 crc32 = new CRC32();
-        crc32.update(headerBytes);
-
-        ByteBuffer bb = ByteBuffer.allocate(SevenZFile.sevenZSignature.length
-                                            + 2 /* version */
-                                            + 4 /* start header CRC */
-                                            + 8 /* next header position */
-                                            + 8 /* next header length */
-                                            + 4 /* next header CRC */)
-            .order(ByteOrder.LITTLE_ENDIAN);
+        
         // signature header
-        channel.position(0);
-        bb.put(SevenZFile.sevenZSignature);
+        file.seek(0);
+        file.write(SevenZFile.sevenZSignature);
         // version
-        bb.put((byte) 0).put((byte) 2);
-
-        // placeholder for start header CRC
-        bb.putInt(0);
-
+        file.write(0);
+        file.write(2);
+        
         // start header
-        bb.putLong(headerPosition - SevenZFile.SIGNATURE_HEADER_SIZE)
-            .putLong(0xffffFFFFL & headerBytes.length)
-            .putInt((int) crc32.getValue());
+        final ByteArrayOutputStream startHeaderBaos = new ByteArrayOutputStream();
+        final DataOutputStream startHeaderStream = new DataOutputStream(startHeaderBaos);
+        startHeaderStream.writeLong(Long.reverseBytes(headerPosition - SevenZFile.SIGNATURE_HEADER_SIZE));
+        startHeaderStream.writeLong(Long.reverseBytes(0xffffFFFFL & headerBytes.length));
         crc32.reset();
-        crc32.update(bb.array(), SevenZFile.sevenZSignature.length + 6, 20);
-        bb.putInt(SevenZFile.sevenZSignature.length + 2, (int) crc32.getValue());
-        bb.flip();
-        channel.write(bb);
+        crc32.update(headerBytes);
+        startHeaderStream.writeInt(Integer.reverseBytes((int)crc32.getValue()));
+        startHeaderStream.flush();
+        final byte[] startHeaderBytes = startHeaderBaos.toByteArray();
+        crc32.reset();
+        crc32.update(startHeaderBytes);
+        file.writeInt(Integer.reverseBytes((int) crc32.getValue()));
+        file.write(startHeaderBytes);
     }
-
+    
     /*
      * Creation of output stream is deferred until data is actually
      * written as some codecs might write header information even for
@@ -302,38 +237,23 @@ public class SevenZOutputFile implements Closeable {
     }
 
     private CountingOutputStream setupFileOutputStream() throws IOException {
-        if (files.isEmpty()) {
-            throw new IllegalStateException("No current 7z entry");
-        }
-
         OutputStream out = new OutputStreamWrapper();
-        final ArrayList<CountingOutputStream> moreStreams = new ArrayList<>();
-        boolean first = true;
-        for (final SevenZMethodConfiguration m : getContentMethods(files.get(files.size() - 1))) {
-            if (!first) {
-                final CountingOutputStream cos = new CountingOutputStream(out);
-                moreStreams.add(cos);
-                out = cos;
-            }
-            out = Coders.addEncoder(out, m.getMethod(), m.getOptions());
-            first = false;
-        }
-        if (!moreStreams.isEmpty()) {
-            additionalCountingStreams = moreStreams.toArray(new CountingOutputStream[moreStreams.size()]);
-        }
-        return new CountingOutputStream(out) {
+        return new CountingOutputStream(Coders
+                                        .addEncoder(out,
+                                                    contentCompression,
+                                                    null)) {
             @Override
             public void write(final int b) throws IOException {
                 super.write(b);
                 crc32.update(b);
             }
-
+    
             @Override
             public void write(final byte[] b) throws IOException {
                 super.write(b);
                 crc32.update(b);
             }
-
+    
             @Override
             public void write(final byte[] b, final int off, final int len)
                 throws IOException {
@@ -343,128 +263,100 @@ public class SevenZOutputFile implements Closeable {
         };
     }
 
-    private Iterable<? extends SevenZMethodConfiguration> getContentMethods(final SevenZArchiveEntry entry) {
-        final Iterable<? extends SevenZMethodConfiguration> ms = entry.getContentMethods();
-        return ms == null ? contentMethods : ms;
-    }
-
     private void writeHeader(final DataOutput header) throws IOException {
         header.write(NID.kHeader);
-
+        
         header.write(NID.kMainStreamsInfo);
         writeStreamsInfo(header);
         writeFilesInfo(header);
         header.write(NID.kEnd);
     }
-
+    
     private void writeStreamsInfo(final DataOutput header) throws IOException {
         if (numNonEmptyStreams > 0) {
             writePackInfo(header);
             writeUnpackInfo(header);
         }
-
+        
         writeSubStreamsInfo(header);
-
+        
         header.write(NID.kEnd);
     }
-
+    
     private void writePackInfo(final DataOutput header) throws IOException {
         header.write(NID.kPackInfo);
-
+        
         writeUint64(header, 0);
         writeUint64(header, 0xffffFFFFL & numNonEmptyStreams);
-
+        
         header.write(NID.kSize);
         for (final SevenZArchiveEntry entry : files) {
             if (entry.hasStream()) {
                 writeUint64(header, entry.getCompressedSize());
             }
         }
-
+        
         header.write(NID.kCRC);
-        header.write(1); // "allAreDefined" == true
+        header.write(1);
         for (final SevenZArchiveEntry entry : files) {
             if (entry.hasStream()) {
                 header.writeInt(Integer.reverseBytes((int) entry.getCompressedCrcValue()));
             }
         }
-
+        
         header.write(NID.kEnd);
     }
-
+    
     private void writeUnpackInfo(final DataOutput header) throws IOException {
         header.write(NID.kUnpackInfo);
-
+        
         header.write(NID.kFolder);
         writeUint64(header, numNonEmptyStreams);
         header.write(0);
-        for (final SevenZArchiveEntry entry : files) {
-            if (entry.hasStream()) {
-                writeFolder(header, entry);
-            }
+        for (int i = 0; i < numNonEmptyStreams; i++) {
+            writeFolder(header);
         }
-
+        
         header.write(NID.kCodersUnpackSize);
         for (final SevenZArchiveEntry entry : files) {
             if (entry.hasStream()) {
-                final long[] moreSizes = additionalSizes.get(entry);
-                if (moreSizes != null) {
-                    for (final long s : moreSizes) {
-                        writeUint64(header, s);
-                    }
-                }
                 writeUint64(header, entry.getSize());
             }
         }
-
+        
         header.write(NID.kCRC);
-        header.write(1); // "allAreDefined" == true
+        header.write(1);
         for (final SevenZArchiveEntry entry : files) {
             if (entry.hasStream()) {
                 header.writeInt(Integer.reverseBytes((int) entry.getCrcValue()));
             }
         }
-
+        
         header.write(NID.kEnd);
     }
-
-    private void writeFolder(final DataOutput header, final SevenZArchiveEntry entry) throws IOException {
-        final ByteArrayOutputStream bos = new ByteArrayOutputStream();
-        int numCoders = 0;
-        for (final SevenZMethodConfiguration m : getContentMethods(entry)) {
-            numCoders++;
-            writeSingleCodec(m, bos);
-        }
-
-        writeUint64(header, numCoders);
-        header.write(bos.toByteArray());
-        for (long i = 0; i < numCoders - 1; i++) {
-            writeUint64(header, i + 1);
-            writeUint64(header, i);
-        }
-    }
-
-    private void writeSingleCodec(final SevenZMethodConfiguration m, final OutputStream bos) throws IOException {
-        final byte[] id = m.getMethod().getId();
-        final byte[] properties = Coders.findByMethod(m.getMethod())
-            .getOptionsAsProperties(m.getOptions());
+    
+    private void writeFolder(final DataOutput header) throws IOException {
+        // one coder
+        writeUint64(header, 1);
+        byte[] id = contentCompression.getId();
+        byte[] properties = contentCompression.getProperties();
 
         int codecFlags = id.length;
         if (properties.length > 0) {
             codecFlags |= 0x20;
         }
-        bos.write(codecFlags);
-        bos.write(id);
+        header.write(codecFlags);
+        header.write(id);
 
         if (properties.length > 0) {
-            bos.write(properties.length);
-            bos.write(properties);
+            header.write(properties.length);
+            header.write(properties);
         }
     }
-
+    
     private void writeSubStreamsInfo(final DataOutput header) throws IOException {
         header.write(NID.kSubStreamsInfo);
-//
+//        
 //        header.write(NID.kCRC);
 //        header.write(1);
 //        for (final SevenZArchiveEntry entry : files) {
@@ -472,13 +364,13 @@ public class SevenZOutputFile implements Closeable {
 //                header.writeInt(Integer.reverseBytes(entry.getCrc()));
 //            }
 //        }
-//
+//        
         header.write(NID.kEnd);
     }
-
+    
     private void writeFilesInfo(final DataOutput header) throws IOException {
         header.write(NID.kFilesInfo);
-
+        
         writeUint64(header, files.size());
 
         writeFileEmptyStreams(header);
@@ -489,9 +381,9 @@ public class SevenZOutputFile implements Closeable {
         writeFileATimes(header);
         writeFileMTimes(header);
         writeFileWindowsAttributes(header);
-        header.write(NID.kEnd);
+        header.write(0);
     }
-
+    
     private void writeFileEmptyStreams(final DataOutput header) throws IOException {
         boolean hasEmptyStreams = false;
         for (final SevenZArchiveEntry entry : files) {
@@ -515,14 +407,14 @@ public class SevenZOutputFile implements Closeable {
             header.write(contents);
         }
     }
-
+    
     private void writeFileEmptyFiles(final DataOutput header) throws IOException {
         boolean hasEmptyFiles = false;
         int emptyStreamCounter = 0;
         final BitSet emptyFiles = new BitSet(0);
-        for (final SevenZArchiveEntry file1 : files) {
-            if (!file1.hasStream()) {
-                final boolean isDir = file1.isDirectory();
+        for (int i = 0; i < files.size(); i++) {
+            if (!files.get(i).hasStream()) {
+                boolean isDir = files.get(i).isDirectory();
                 emptyFiles.set(emptyStreamCounter++, !isDir);
                 hasEmptyFiles |= !isDir;
             }
@@ -538,14 +430,14 @@ public class SevenZOutputFile implements Closeable {
             header.write(contents);
         }
     }
-
+    
     private void writeFileAntiItems(final DataOutput header) throws IOException {
         boolean hasAntiItems = false;
         final BitSet antiItems = new BitSet(0);
         int antiItemCounter = 0;
-        for (final SevenZArchiveEntry file1 : files) {
-            if (!file1.hasStream()) {
-                final boolean isAnti = file1.isAntiItem();
+        for (int i = 0; i < files.size(); i++) {
+            if (!files.get(i).hasStream()) {
+                boolean isAnti = files.get(i).isAntiItem();
                 antiItems.set(antiItemCounter++, isAnti);
                 hasAntiItems |= isAnti;
             }
@@ -561,10 +453,10 @@ public class SevenZOutputFile implements Closeable {
             header.write(contents);
         }
     }
-
+    
     private void writeFileNames(final DataOutput header) throws IOException {
         header.write(NID.kName);
-
+        
         final ByteArrayOutputStream baos = new ByteArrayOutputStream();
         final DataOutputStream out = new DataOutputStream(baos);
         out.write(0);
@@ -598,7 +490,7 @@ public class SevenZOutputFile implements Closeable {
                 }
                 writeBits(out, cTimes, files.size());
             } else {
-                out.write(1); // "allAreDefined" == true
+                out.write(1);
             }
             out.write(0);
             for (final SevenZArchiveEntry entry : files) {
@@ -634,7 +526,7 @@ public class SevenZOutputFile implements Closeable {
                 }
                 writeBits(out, aTimes, files.size());
             } else {
-                out.write(1); // "allAreDefined" == true
+                out.write(1);
             }
             out.write(0);
             for (final SevenZArchiveEntry entry : files) {
@@ -670,7 +562,7 @@ public class SevenZOutputFile implements Closeable {
                 }
                 writeBits(out, mTimes, files.size());
             } else {
-                out.write(1); // "allAreDefined" == true
+                out.write(1);
             }
             out.write(0);
             for (final SevenZArchiveEntry entry : files) {
@@ -706,7 +598,7 @@ public class SevenZOutputFile implements Closeable {
                 }
                 writeBits(out, attributes, files.size());
             } else {
-                out.write(1); // "allAreDefined" == true
+                out.write(1);
             }
             out.write(0);
             for (final SevenZArchiveEntry entry : files) {
@@ -756,48 +648,30 @@ public class SevenZOutputFile implements Closeable {
         }
     }
 
-    private static <T> Iterable<T> reverse(final Iterable<T> i) {
-        final LinkedList<T> l = new LinkedList<>();
-        for (final T t : i) {
-            l.addFirst(t);
-        }
-        return l;
-    }
-
     private class OutputStreamWrapper extends OutputStream {
-        private static final int BUF_SIZE = 8192;
-        private final ByteBuffer buffer = ByteBuffer.allocate(BUF_SIZE);
         @Override
         public void write(final int b) throws IOException {
-            buffer.clear();
-            buffer.put((byte) b).flip();
-            channel.write(buffer);
+            file.write(b);
             compressedCrc32.update(b);
             fileBytesWritten++;
         }
-
+    
         @Override
         public void write(final byte[] b) throws IOException {
             OutputStreamWrapper.this.write(b, 0, b.length);
         }
-
+    
         @Override
         public void write(final byte[] b, final int off, final int len)
             throws IOException {
-            if (len > BUF_SIZE) {
-                channel.write(ByteBuffer.wrap(b, off, len));
-            } else {
-                buffer.clear();
-                buffer.put(b, off, len).flip();
-                channel.write(buffer);
-            }
+            file.write(b, off, len);
             compressedCrc32.update(b, off, len);
             fileBytesWritten += len;
         }
 
         @Override
         public void flush() throws IOException {
-            // no reason to flush the channel
+            // no reason to flush a RandomAccessFile
         }
 
         @Override
