@@ -20,45 +20,36 @@ package org.apache.commons.compress.archivers.zip;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.nio.ByteOrder;
 
-import org.apache.commons.compress.compressors.lzw.LZWInputStream;
+import org.apache.commons.compress.compressors.z._internal_.InternalLZWInputStream;
 
 /**
  * Input stream that decompresses ZIP method 1 (unshrinking). A variation of the LZW algorithm, with some twists.
  * @NotThreadSafe
  * @since 1.7
  */
-class UnshrinkingInputStream extends LZWInputStream {
+class UnshrinkingInputStream extends InternalLZWInputStream {
     private static final int MAX_CODE_SIZE = 13;
     private static final int MAX_TABLE_SIZE = 1 << MAX_CODE_SIZE;
     private final boolean[] isUsed;
     
-    /**
-     * IOException is not actually thrown!
-     * 
-     * @param inputStream
-     * @throws IOException IOException is not actually thrown!
-     */
-    public UnshrinkingInputStream(final InputStream inputStream) throws IOException {
-        super(inputStream, ByteOrder.LITTLE_ENDIAN);
-        setClearCode(DEFAULT_CODE_SIZE);
+    public UnshrinkingInputStream(InputStream inputStream) throws IOException {
+        super(inputStream);
+        setClearCode(codeSize);
         initializeTables(MAX_CODE_SIZE);
-        isUsed = new boolean[getPrefixesLength()];
+        isUsed = new boolean[prefixes.length];
         for (int i = 0; i < (1 << 8); i++) {
             isUsed[i] = true;
         }
-        setTableSize(getClearCode() + 1);
+        tableSize = clearCode + 1;
     }
 
     @Override
-    protected int addEntry(final int previousCode, final byte character) throws IOException {
-        int tableSize = getTableSize();
+    protected int addEntry(int previousCode, byte character) throws IOException {
         while ((tableSize < MAX_TABLE_SIZE) && isUsed[tableSize]) {
             tableSize++;
         }
-        setTableSize(tableSize);
-        final int idx = addEntry(previousCode, character, MAX_TABLE_SIZE);
+        int idx = addEntry(previousCode, character, MAX_TABLE_SIZE);
         if (idx >= 0) {
             isUsed[idx] = true;
         }
@@ -68,14 +59,14 @@ class UnshrinkingInputStream extends LZWInputStream {
     private void partialClear() {
         final boolean[] isParent = new boolean[MAX_TABLE_SIZE];
         for (int i = 0; i < isUsed.length; i++) {
-            if (isUsed[i] && getPrefix(i) != UNUSED_PREFIX) {
-                isParent[getPrefix(i)] = true;
+            if (isUsed[i] && prefixes[i] != -1) {
+                isParent[prefixes[i]] = true;
             }
         }
-        for (int i = getClearCode() + 1; i < isParent.length; i++) {
+        for (int i = clearCode + 1; i < isParent.length; i++) {
             if (!isParent[i]) {
                 isUsed[i] = false;
-                setPrefix(i, UNUSED_PREFIX);
+                prefixes[i] = -1;
             }
         }
     }
@@ -97,19 +88,19 @@ class UnshrinkingInputStream extends LZWInputStream {
         final int code = readNextCode();
         if (code < 0) {
             return -1;
-        } else if (code == getClearCode()) {
+        } else if (code == clearCode) {
             final int subCode = readNextCode();
             if (subCode < 0) {
                 throw new IOException("Unexpected EOF;");
             } else if (subCode == 1) {
-                if (getCodeSize() < MAX_CODE_SIZE) {
-                    incrementCodeSize();
+                if (codeSize < MAX_CODE_SIZE) {
+                    codeSize++;
                 } else {
                     throw new IOException("Attempt to increase code size beyond maximum");
                 }
             } else if (subCode == 2) {
                 partialClear();
-                setTableSize(getClearCode() + 1);
+                tableSize = clearCode + 1;
             } else {
                 throw new IOException("Invalid clear code subcode " + subCode);
             }
