@@ -19,6 +19,7 @@
 package org.apache.commons.compress.changes;
 
 import java.io.InputStream;
+import java.util.Objects;
 
 import org.apache.commons.compress.archivers.ArchiveEntry;
 
@@ -42,15 +43,13 @@ class Change {
     static final int TYPE_DELETE_DIR = 4;
 
     /**
-     * Constructor. Takes the filename of the file to be deleted
+     * Constructor. Takes the file name of the file to be deleted
      * from the stream as argument.
-     * @param pFilename the filename of the file to delete
+     * @param fileName the file name of the file to delete
      */
-    Change(final String pFilename, final int type) {
-        if(pFilename == null) {
-            throw new NullPointerException();
-        }
-        this.targetFile = pFilename;
+    Change(final String fileName, final int type) {
+        Objects.requireNonNull(fileName, "fileName");
+        this.targetFile = fileName;
         this.type = type;
         this.input = null;
         this.entry = null;
@@ -60,15 +59,14 @@ class Change {
     /**
      * Construct a change which adds an entry.
      *
-     * @param pEntry the entry details
-     * @param pInput the InputStream for the entry data
+     * @param archiveEntry the entry details
+     * @param inputStream the InputStream for the entry data
      */
-    Change(final ArchiveEntry pEntry, final InputStream pInput, final boolean replace) {
-        if(pEntry == null || pInput == null) {
-            throw new NullPointerException();
-        }
-        this.entry = pEntry;
-        this.input = pInput;
+    Change(final ArchiveEntry archiveEntry, final InputStream inputStream, final boolean replace) {
+        Objects.requireNonNull(archiveEntry, "archiveEntry");
+        Objects.requireNonNull(inputStream, "inputStream");
+        this.entry = archiveEntry;
+        this.input = inputStream;
         type = TYPE_ADD;
         targetFile = null;
         this.replaceMode = replace;
