@@ -31,23 +31,34 @@ import java.util.Set;
 import java.util.SortedMap;
 import java.util.TreeMap;
 
+import org.apache.commons.compress.compressors.brotli.BrotliCompressorInputStream;
+import org.apache.commons.compress.compressors.brotli.BrotliUtils;
 import org.apache.commons.compress.compressors.bzip2.BZip2CompressorInputStream;
 import org.apache.commons.compress.compressors.bzip2.BZip2CompressorOutputStream;
 import org.apache.commons.compress.compressors.deflate.DeflateCompressorInputStream;
 import org.apache.commons.compress.compressors.deflate.DeflateCompressorOutputStream;
+import org.apache.commons.compress.compressors.deflate64.Deflate64CompressorInputStream;
 import org.apache.commons.compress.compressors.gzip.GzipCompressorInputStream;
 import org.apache.commons.compress.compressors.gzip.GzipCompressorOutputStream;
+import org.apache.commons.compress.compressors.lz4.BlockLZ4CompressorInputStream;
+import org.apache.commons.compress.compressors.lz4.BlockLZ4CompressorOutputStream;
+import org.apache.commons.compress.compressors.lz4.FramedLZ4CompressorInputStream;
+import org.apache.commons.compress.compressors.lz4.FramedLZ4CompressorOutputStream;
 import org.apache.commons.compress.compressors.lzma.LZMACompressorInputStream;
 import org.apache.commons.compress.compressors.lzma.LZMACompressorOutputStream;
 import org.apache.commons.compress.compressors.lzma.LZMAUtils;
 import org.apache.commons.compress.compressors.pack200.Pack200CompressorInputStream;
 import org.apache.commons.compress.compressors.pack200.Pack200CompressorOutputStream;
 import org.apache.commons.compress.compressors.snappy.FramedSnappyCompressorInputStream;
+import org.apache.commons.compress.compressors.snappy.FramedSnappyCompressorOutputStream;
 import org.apache.commons.compress.compressors.snappy.SnappyCompressorInputStream;
 import org.apache.commons.compress.compressors.xz.XZCompressorInputStream;
 import org.apache.commons.compress.compressors.xz.XZCompressorOutputStream;
 import org.apache.commons.compress.compressors.xz.XZUtils;
 import org.apache.commons.compress.compressors.z.ZCompressorInputStream;
+import org.apache.commons.compress.compressors.zstandard.ZstdCompressorInputStream;
+import org.apache.commons.compress.compressors.zstandard.ZstdCompressorOutputStream;
+import org.apache.commons.compress.compressors.zstandard.ZstdUtils;
 import org.apache.commons.compress.utils.IOUtils;
 import org.apache.commons.compress.utils.Lists;
 import org.apache.commons.compress.utils.ServiceLoaderIterator;
@@ -59,27 +70,27 @@ import org.apache.commons.compress.utils.Sets;
  * implementations you should extend CompressorStreamFactory and override the
  * appropriate methods (and call their implementation from super of course).
  * </p>
- * 
+ *
  * Example (Compressing a file):
- * 
+ *
  * <pre>
- * final OutputStream out = new FileOutputStream(output);
+ * final OutputStream out = Files.newOutputStream(output.toPath());
  * CompressorOutputStream cos = new CompressorStreamFactory()
  *         .createCompressorOutputStream(CompressorStreamFactory.BZIP2, out);
- * IOUtils.copy(new FileInputStream(input), cos);
+ * IOUtils.copy(Files.newInputStream(input.toPath()), cos);
  * cos.close();
  * </pre>
- * 
+ *
  * Example (Decompressing a file):
- * 
+ *
  * <pre>
- * final InputStream is = new FileInputStream(input);
+ * final InputStream is = Files.newInputStream(input.toPath());
  * CompressorInputStream in = new CompressorStreamFactory().createCompressorInputStream(CompressorStreamFactory.BZIP2,
  *         is);
- * IOUtils.copy(in, new FileOutputStream(output));
+ * IOUtils.copy(in, Files.newOutputStream(output.toPath()));
  * in.close();
  * </pre>
- * 
+ *
  * @Immutable provided that the deprecated method setDecompressConcatenated is
  *            not used.
  * @ThreadSafe even if the deprecated method setDecompressConcatenated is used
@@ -88,18 +99,28 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
 
     private static final CompressorStreamFactory SINGLETON = new CompressorStreamFactory();
 
+
+
+    /**
+     * Constant (value {@value}) used to identify the BROTLI compression
+     * algorithm.
+     *
+     * @since 1.14
+     */
+    public static final String BROTLI = "br";
+
     /**
      * Constant (value {@value}) used to identify the BZIP2 compression
      * algorithm.
-     * 
+     *
      * @since 1.1
      */
     public static final String BZIP2 = "bzip2";
 
     /**
      * Constant (value {@value}) used to identify the GZIP compression
-     * algorithm. Not supported as an output stream type.
-     * 
+     * algorithm.
+     *
      * @since 1.1
      */
     public static final String GZIP = "gz";
@@ -107,30 +128,29 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
     /**
      * Constant (value {@value}) used to identify the PACK200 compression
      * algorithm.
-     * 
+     *
      * @since 1.3
      */
     public static final String PACK200 = "pack200";
 
     /**
      * Constant (value {@value}) used to identify the XZ compression method.
-     * 
+     *
      * @since 1.4
      */
     public static final String XZ = "xz";
 
     /**
      * Constant (value {@value}) used to identify the LZMA compression method.
-     * Not supported as an output stream type.
-     * 
+     *
      * @since 1.6
      */
     public static final String LZMA = "lzma";
 
     /**
      * Constant (value {@value}) used to identify the "framed" Snappy
-     * compression method. Not supported as an output stream type.
-     * 
+     * compression method.
+     *
      * @since 1.7
      */
     public static final String SNAPPY_FRAMED = "snappy-framed";
@@ -138,7 +158,7 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
     /**
      * Constant (value {@value}) used to identify the "raw" Snappy compression
      * method. Not supported as an output stream type.
-     * 
+     *
      * @since 1.7
      */
     public static final String SNAPPY_RAW = "snappy-raw";
@@ -146,18 +166,57 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
     /**
      * Constant (value {@value}) used to identify the traditional Unix compress
      * method. Not supported as an output stream type.
-     * 
+     *
      * @since 1.7
      */
     public static final String Z = "z";
 
     /**
      * Constant (value {@value}) used to identify the Deflate compress method.
-     * 
+     *
      * @since 1.9
      */
     public static final String DEFLATE = "deflate";
 
+    /**
+     * Constant (value {@value}) used to identify the Deflate64 compress method.
+     *
+     * @since 1.16
+     */
+    public static final String DEFLATE64 = "deflate64";
+
+    /**
+     * Constant (value {@value}) used to identify the block LZ4
+     * compression method.
+     *
+     * @since 1.14
+     */
+    public static final String LZ4_BLOCK = "lz4-block";
+
+    /**
+     * Constant (value {@value}) used to identify the frame LZ4
+     * compression method.
+     *
+     * @since 1.14
+     */
+    public static final String LZ4_FRAMED = "lz4-framed";
+
+    /**
+     * Constant (value {@value}) used to identify the Zstandard compression
+     * algorithm. Not supported as an output stream type.
+     *
+     * @since 1.16
+     */
+    public static final String ZSTANDARD = "zstd";
+
+    private static final String YOU_NEED_BROTLI_DEC = youNeed("Google Brotli Dec", "https://github.com/google/brotli/");
+    private static final String YOU_NEED_XZ_JAVA = youNeed("XZ for Java", "https://tukaani.org/xz/java.html");
+    private static final String YOU_NEED_ZSTD_JNI = youNeed("Zstd JNI", "https://github.com/luben/zstd-jni");
+
+    private static String youNeed(String name, String url) {
+        return " In addition to Apache Commons Compress you need the " + name + " library - see " + url;
+    }
+
     /**
      * Constructs a new sorted map from input stream provider names to provider
      * objects.
@@ -243,7 +302,11 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
     private static ArrayList<CompressorStreamProvider> findCompressorStreamProviders() {
         return Lists.newArrayList(serviceLoaderIterator());
     }
-    
+
+    public static String getBrotli() {
+        return BROTLI;
+    }
+
     public static String getBzip2() {
         return BZIP2;
     }
@@ -252,6 +315,14 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
         return DEFLATE;
     }
 
+    /**
+     * @since 1.16
+     * @return the constant {@link #DEFLATE64}
+     */
+    public static String getDeflate64() {
+        return DEFLATE64;
+    }
+
     public static String getGzip() {
         return GZIP;
     }
@@ -284,6 +355,18 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
         return Z;
     }
 
+    public static String getLZ4Framed() {
+        return LZ4_FRAMED;
+    }
+
+    public static String getLZ4Block() {
+        return LZ4_BLOCK;
+    }
+
+    public static String getZstandard() {
+        return ZSTANDARD;
+    }
+
     static void putAll(final Set<String> names, final CompressorStreamProvider provider,
             final TreeMap<String, CompressorStreamProvider> map) {
         for (final String name : names) {
@@ -313,7 +396,7 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
     private SortedMap<String, CompressorStreamProvider> compressorInputStreamProviders;
 
     private SortedMap<String, CompressorStreamProvider> compressorOutputStreamProviders;
-    
+
     /**
      * If true, decompress until the end of the input. If false, stop after the
      * first stream and leave the input position to point to the next byte after
@@ -321,16 +404,43 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
      */
     private volatile boolean decompressConcatenated = false;
 
+    private final int memoryLimitInKb;
     /**
      * Create an instance with the decompress Concatenated option set to false.
      */
     public CompressorStreamFactory() {
         this.decompressUntilEOF = null;
+        this.memoryLimitInKb = -1;
     }
 
     /**
      * Create an instance with the provided decompress Concatenated option.
-     * 
+     *
+     * @param decompressUntilEOF
+     *            if true, decompress until the end of the input; if false, stop
+     *            after the first stream and leave the input position to point
+     *            to the next byte after the stream. This setting applies to the
+     *            gzip, bzip2 and xz formats only.
+     *
+     * @param memoryLimitInKb
+     *            Some streams require allocation of potentially significant
+     *            byte arrays/tables, and they can offer checks to prevent OOMs
+     *            on corrupt files.  Set the maximum allowed memory allocation in KBs.
+     *
+     * @since 1.14
+     */
+    public CompressorStreamFactory(final boolean decompressUntilEOF, final int memoryLimitInKb) {
+        this.decompressUntilEOF = decompressUntilEOF;
+        // Also copy to existing variable so can continue to use that as the
+        // current value
+        this.decompressConcatenated = decompressUntilEOF;
+        this.memoryLimitInKb = memoryLimitInKb;
+    }
+
+
+    /**
+     * Create an instance with the provided decompress Concatenated option.
+     *
      * @param decompressUntilEOF
      *            if true, decompress until the end of the input; if false, stop
      *            after the first stream and leave the input position to point
@@ -339,17 +449,86 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
      * @since 1.10
      */
     public CompressorStreamFactory(final boolean decompressUntilEOF) {
-        this.decompressUntilEOF = Boolean.valueOf(decompressUntilEOF);
-        // Also copy to existing variable so can continue to use that as the
-        // current value
-        this.decompressConcatenated = decompressUntilEOF;
+        this(decompressUntilEOF, -1);
     }
 
+    /**
+     * Try to detect the type of compressor stream.
+     *
+     * @param inputStream input stream
+     * @return type of compressor stream detected
+     * @throws CompressorException if no compressor stream type was detected
+     *                             or if something else went wrong
+     * @throws IllegalArgumentException if stream is null or does not support mark
+     *
+     * @since 1.14
+     */
+    public static String detect(final InputStream inputStream) throws CompressorException {
+        if (inputStream == null) {
+            throw new IllegalArgumentException("Stream must not be null.");
+        }
+
+        if (!inputStream.markSupported()) {
+            throw new IllegalArgumentException("Mark is not supported.");
+        }
+
+        final byte[] signature = new byte[12];
+        inputStream.mark(signature.length);
+        int signatureLength = -1;
+        try {
+            signatureLength = IOUtils.readFully(inputStream, signature);
+            inputStream.reset();
+        } catch (IOException e) {
+            throw new CompressorException("IOException while reading signature.", e);
+        }
+
+        if (BZip2CompressorInputStream.matches(signature, signatureLength)) {
+            return BZIP2;
+        }
+
+        if (GzipCompressorInputStream.matches(signature, signatureLength)) {
+            return GZIP;
+        }
+
+        if (Pack200CompressorInputStream.matches(signature, signatureLength)) {
+            return PACK200;
+        }
+
+        if (FramedSnappyCompressorInputStream.matches(signature, signatureLength)) {
+            return SNAPPY_FRAMED;
+        }
+
+        if (ZCompressorInputStream.matches(signature, signatureLength)) {
+            return Z;
+        }
+
+        if (DeflateCompressorInputStream.matches(signature, signatureLength)) {
+            return DEFLATE;
+        }
+
+        if (XZUtils.matches(signature, signatureLength)) {
+            return XZ;
+        }
+
+        if (LZMAUtils.matches(signature, signatureLength)) {
+            return LZMA;
+        }
+
+        if (FramedLZ4CompressorInputStream.matches(signature, signatureLength)) {
+            return LZ4_FRAMED;
+        }
+
+        if (ZstdUtils.matches(signature, signatureLength)) {
+            return ZSTANDARD;
+        }
+
+        throw new CompressorException("No Compressor found for the stream signature.");
+    }
     /**
      * Create an compressor input stream from an input stream, autodetecting the
      * compressor type from the first few bytes of the stream. The InputStream
      * must support marks, like BufferedInputStream.
-     * 
+     *
      * @param in
      *            the input stream
      * @return the compressor input stream
@@ -360,73 +539,27 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
      * @since 1.1
      */
     public CompressorInputStream createCompressorInputStream(final InputStream in) throws CompressorException {
-        if (in == null) {
-            throw new IllegalArgumentException("Stream must not be null.");
-        }
-
-        if (!in.markSupported()) {
-            throw new IllegalArgumentException("Mark is not supported.");
-        }
-
-        final byte[] signature = new byte[12];
-        in.mark(signature.length);
-        try {
-            final int signatureLength = IOUtils.readFully(in, signature);
-            in.reset();
-
-            if (BZip2CompressorInputStream.matches(signature, signatureLength)) {
-                return new BZip2CompressorInputStream(in, decompressConcatenated);
-            }
-
-            if (GzipCompressorInputStream.matches(signature, signatureLength)) {
-                return new GzipCompressorInputStream(in, decompressConcatenated);
-            }
-
-            if (Pack200CompressorInputStream.matches(signature, signatureLength)) {
-                return new Pack200CompressorInputStream(in);
-            }
-
-            if (FramedSnappyCompressorInputStream.matches(signature, signatureLength)) {
-                return new FramedSnappyCompressorInputStream(in);
-            }
-
-            if (ZCompressorInputStream.matches(signature, signatureLength)) {
-                return new ZCompressorInputStream(in);
-            }
-
-            if (DeflateCompressorInputStream.matches(signature, signatureLength)) {
-                return new DeflateCompressorInputStream(in);
-            }
-
-            if (XZUtils.matches(signature, signatureLength) && XZUtils.isXZCompressionAvailable()) {
-                return new XZCompressorInputStream(in, decompressConcatenated);
-            }
-
-            if (LZMAUtils.matches(signature, signatureLength) && LZMAUtils.isLZMACompressionAvailable()) {
-                return new LZMACompressorInputStream(in);
-            }
-
-        } catch (final IOException e) {
-            throw new CompressorException("Failed to detect Compressor from InputStream.", e);
-        }
-
-        throw new CompressorException("No Compressor found for the stream signature.");
+        return createCompressorInputStream(detect(in), in);
     }
 
     /**
      * Creates a compressor input stream from a compressor name and an input
      * stream.
-     * 
+     *
      * @param name
      *            of the compressor, i.e. {@value #GZIP}, {@value #BZIP2},
      *            {@value #XZ}, {@value #LZMA}, {@value #PACK200},
-     *            {@value #SNAPPY_RAW}, {@value #SNAPPY_FRAMED}, {@value #Z} or
-     *            {@value #DEFLATE}
+     *            {@value #SNAPPY_RAW}, {@value #SNAPPY_FRAMED}, {@value #Z},
+     *            {@value #LZ4_BLOCK}, {@value #LZ4_FRAMED}, {@value #ZSTANDARD},
+     *            {@value #DEFLATE64}
+     *            or {@value #DEFLATE}
      * @param in
      *            the input stream
      * @return compressor input stream
      * @throws CompressorException
-     *             if the compressor name is not known
+     *             if the compressor name is not known or not available,
+     *             or if there's an IOException or MemoryLimitException thrown
+     *             during initialization
      * @throws IllegalArgumentException
      *             if the name or input stream is null
      */
@@ -452,12 +585,32 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
                 return new BZip2CompressorInputStream(in, actualDecompressConcatenated);
             }
 
+            if (BROTLI.equalsIgnoreCase(name)) {
+                if (!BrotliUtils.isBrotliCompressionAvailable()) {
+                    throw new CompressorException("Brotli compression is not available." + YOU_NEED_BROTLI_DEC);
+                }
+                return new BrotliCompressorInputStream(in);
+            }
+
             if (XZ.equalsIgnoreCase(name)) {
-                return new XZCompressorInputStream(in, actualDecompressConcatenated);
+                if (!XZUtils.isXZCompressionAvailable()) {
+                    throw new CompressorException("XZ compression is not available." + YOU_NEED_XZ_JAVA);
+                }
+                return new XZCompressorInputStream(in, actualDecompressConcatenated, memoryLimitInKb);
+            }
+
+            if (ZSTANDARD.equalsIgnoreCase(name)) {
+                if (!ZstdUtils.isZstdCompressionAvailable()) {
+                    throw new CompressorException("Zstandard compression is not available." + YOU_NEED_ZSTD_JNI);
+                }
+                return new ZstdCompressorInputStream(in);
             }
 
             if (LZMA.equalsIgnoreCase(name)) {
-                return new LZMACompressorInputStream(in);
+                if (!LZMAUtils.isLZMACompressionAvailable()) {
+                    throw new CompressorException("LZMA compression is not available" + YOU_NEED_XZ_JAVA);
+                }
+                return new LZMACompressorInputStream(in, memoryLimitInKb);
             }
 
             if (PACK200.equalsIgnoreCase(name)) {
@@ -473,13 +626,25 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
             }
 
             if (Z.equalsIgnoreCase(name)) {
-                return new ZCompressorInputStream(in);
+                return new ZCompressorInputStream(in, memoryLimitInKb);
             }
 
             if (DEFLATE.equalsIgnoreCase(name)) {
                 return new DeflateCompressorInputStream(in);
             }
 
+            if (DEFLATE64.equalsIgnoreCase(name)) {
+                return new Deflate64CompressorInputStream(in);
+            }
+
+            if (LZ4_BLOCK.equalsIgnoreCase(name)) {
+                return new BlockLZ4CompressorInputStream(in);
+            }
+
+            if (LZ4_FRAMED.equalsIgnoreCase(name)) {
+                return new FramedLZ4CompressorInputStream(in, actualDecompressConcatenated);
+            }
+
         } catch (final IOException e) {
             throw new CompressorException("Could not create CompressorInputStream.", e);
         }
@@ -487,17 +652,19 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
         if (compressorStreamProvider != null) {
             return compressorStreamProvider.createCompressorInputStream(name, in, actualDecompressConcatenated);
         }
-        
+
         throw new CompressorException("Compressor: " + name + " not found.");
     }
 
     /**
      * Creates an compressor output stream from an compressor name and an output
      * stream.
-     * 
+     *
      * @param name
      *            the compressor name, i.e. {@value #GZIP}, {@value #BZIP2},
-     *            {@value #XZ}, {@value #PACK200} or {@value #DEFLATE}
+     *            {@value #XZ}, {@value #PACK200}, {@value #SNAPPY_FRAMED},
+     *            {@value #LZ4_BLOCK}, {@value #LZ4_FRAMED}, {@value #ZSTANDARD}
+     *            or {@value #DEFLATE}
      * @param out
      *            the output stream
      * @return the compressor output stream
@@ -539,6 +706,21 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
                 return new DeflateCompressorOutputStream(out);
             }
 
+            if (SNAPPY_FRAMED.equalsIgnoreCase(name)) {
+                return new FramedSnappyCompressorOutputStream(out);
+            }
+
+            if (LZ4_BLOCK.equalsIgnoreCase(name)) {
+                return new BlockLZ4CompressorOutputStream(out);
+            }
+
+            if (LZ4_FRAMED.equalsIgnoreCase(name)) {
+                return new FramedLZ4CompressorOutputStream(out);
+            }
+
+            if (ZSTANDARD.equalsIgnoreCase(name)) {
+                return new ZstdCompressorOutputStream(out);
+            }
         } catch (final IOException e) {
             throw new CompressorException("Could not create CompressorOutputStream", e);
         }
@@ -576,12 +758,13 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
 
     @Override
     public Set<String> getInputStreamCompressorNames() {
-        return Sets.newHashSet(GZIP, BZIP2, XZ, LZMA, PACK200, SNAPPY_RAW, SNAPPY_FRAMED, Z, DEFLATE);
+        return Sets.newHashSet(GZIP, BROTLI, BZIP2, XZ, LZMA, PACK200, DEFLATE, SNAPPY_RAW, SNAPPY_FRAMED, Z, LZ4_BLOCK,
+            LZ4_FRAMED, ZSTANDARD, DEFLATE64);
     }
 
     @Override
     public Set<String> getOutputStreamCompressorNames() {
-        return Sets.newHashSet(GZIP, BZIP2, XZ, LZMA, PACK200, DEFLATE);
+        return Sets.newHashSet(GZIP, BZIP2, XZ, LZMA, PACK200, DEFLATE, SNAPPY_FRAMED, LZ4_BLOCK, LZ4_FRAMED, ZSTANDARD);
     }
 
     /**
@@ -610,5 +793,5 @@ public class CompressorStreamFactory implements CompressorStreamProvider {
         }
         this.decompressConcatenated = decompressConcatenated;
     }
-    
+
 }
