@@ -143,7 +143,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
             count(r < 0 ? -1 : 1);
             return r;
         }
-        throw new IOException("stream closed");
+        throw new IOException("Stream closed");
     }
 
     /*
@@ -165,7 +165,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
                                                 + len + ") > dest.length(" + dest.length + ").");
         }
         if (this.bin == null) {
-            throw new IOException("stream closed");
+            throw new IOException("Stream closed");
         }
         if (len == 0) {
             return 0;
@@ -315,7 +315,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
             magic5 != 0x59 // 'Y'
             ) {
             this.currentState = EOF;
-            throw new IOException("bad block header");
+            throw new IOException("Bad block header");
         }
         this.storedBlockCRC = bsGetInt(bin);
         this.blockRandomised = bsR(bin, 1) == 1;
@@ -390,7 +390,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
     private static int bsR(BitInputStream bin, final int n) throws IOException {
         long thech = bin.readBits(n);
         if (thech < 0) {
-            throw new IOException("unexpected end of stream");
+            throw new IOException("Unexpected end of stream");
         }
         return (int) thech;
     }
@@ -494,18 +494,27 @@ public class BZip2CompressorInputStream extends CompressorInputStream
         final int alphaSize = this.nInUse + 2;
         /* Now the selectors */
         final int nGroups = bsR(bin, 3);
-        final int nSelectors = bsR(bin, 15);
+        final int selectors = bsR(bin, 15);
+        if (selectors < 0) {
+            throw new IOException("Corrupted input, nSelectors value negative");
+        }
         checkBounds(alphaSize, MAX_ALPHA_SIZE + 1, "alphaSize");
         checkBounds(nGroups, N_GROUPS + 1, "nGroups");
-        checkBounds(nSelectors, MAX_SELECTORS + 1, "nSelectors");
 
-        for (int i = 0; i < nSelectors; i++) {
+        // Don't fail on nSelectors overflowing boundaries but discard the values in overflow
+        // See https://gnu.wildebeest.org/blog/mjw/2019/08/02/bzip2-and-the-cve-that-wasnt/
+        // and https://sourceware.org/ml/bzip2-devel/2019-q3/msg00007.html
+
+        for (int i = 0; i < selectors; i++) {
             int j = 0;
             while (bsGetBit(bin)) {
                 j++;
             }
-            selectorMtf[i] = (byte) j;
+            if (i < MAX_SELECTORS) {
+                selectorMtf[i] = (byte) j;
+            }
         }
+        final int nSelectors = selectors > MAX_SELECTORS ? MAX_SELECTORS : selectors;
 
         /* Undo the MTF values for the selectors. */
         for (int v = nGroups; --v >= 0;) {
@@ -661,12 +670,12 @@ public class BZip2CompressorInputStream extends CompressorInputStream
                 Arrays.fill(ll8, from, lastShadow + 1, ch);
 
                 if (lastShadow >= limitLast) {
-                    throw new IOException("block overrun while expanding RLE in MTF, "
+                    throw new IOException("Block overrun while expanding RLE in MTF, "
                         + lastShadow + " exceeds " + limitLast);
                 }
             } else {
                 if (++lastShadow >= limitLast) {
-                    throw new IOException("block overrun in MTF, "
+                    throw new IOException("Block overrun in MTF, "
                         + lastShadow + " exceeds " + limitLast);
                 }
                 checkBounds(nextSym, 256 + 1, "nextSym");
@@ -762,7 +771,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream
         }
 
         if ((this.origPtr < 0) || (this.origPtr >= tt.length)) {
-            throw new IOException("stream corrupted");
+            throw new IOException("Stream corrupted");
         }
 
         this.su_tPos = tt[this.origPtr];
