@@ -286,6 +286,9 @@ public class GzipCompressorInputStream extends CompressorInputStream
      */
     @Override
     public int read(final byte[] b, int off, int len) throws IOException {
+        if (len == 0) {
+            return 0;
+        }
         if (endReached) {
             return -1;
         }
@@ -322,9 +325,6 @@ public class GzipCompressorInputStream extends CompressorInputStream
             if (inf.finished()) {
                 // We may have read too many bytes. Rewind the read
                 // position to match the actual amount used.
-                //
-                // NOTE: The "if" is there just in case. Since we used
-                // in.mark earlier, it should always skip enough.
                 in.reset();
 
                 final int skipAmount = bufUsed - inf.getRemaining();
