@@ -20,6 +20,7 @@ package org.apache.commons.compress.compressors.lz77support;
 
 import java.io.IOException;
 import java.util.Arrays;
+import java.util.Objects;
 
 /**
  * Helper class for compression algorithms that use the ideas of LZ77.
@@ -255,12 +256,9 @@ public class LZ77Compressor {
      * @throws NullPointerException if either parameter is <code>null</code>
      */
     public LZ77Compressor(Parameters params, Callback callback) {
-        if (params == null) {
-            throw new NullPointerException("params must not be null");
-        }
-        if (callback == null) {
-            throw new NullPointerException("callback must not be null");
-        }
+        Objects.requireNonNull(params, "params");
+        Objects.requireNonNull(callback, "callback");
+        
         this.params = params;
         this.callback = callback;
 
