@@ -18,56 +18,79 @@
  */
 package org.apache.commons.compress.archivers.dump;
 
-import static org.junit.Assert.assertArrayEquals;
+import static org.junit.Assert.*;
 
 import java.io.FileInputStream;
 import java.io.InputStream;
 
 import org.apache.commons.compress.AbstractTestCase;
+import org.apache.commons.compress.archivers.ArchiveEntry;
 import org.apache.commons.compress.archivers.ArchiveException;
+import org.apache.commons.compress.utils.IOUtils;
+import org.junit.Test;
 
 public class DumpArchiveInputStreamTest extends AbstractTestCase {
 
+    @Test
     public void testNotADumpArchive() throws Exception {
-        FileInputStream is = new FileInputStream(getFile("bla.zip"));
-        try {
-            new DumpArchiveInputStream(is);
+        try (FileInputStream is = new FileInputStream(getFile("bla.zip"))) {
+            new DumpArchiveInputStream(is).close();
             fail("expected an exception");
-        } catch (ArchiveException ex) {
+        } catch (final ArchiveException ex) {
             // expected
             assertTrue(ex.getCause() instanceof ShortFileException);
-        } finally {
-            is.close();
         }
     }
 
+    @Test
     public void testNotADumpArchiveButBigEnough() throws Exception {
-        FileInputStream is = new FileInputStream(getFile("zip64support.tar.bz2"));
-        try {
-            new DumpArchiveInputStream(is);
+        try (FileInputStream is = new FileInputStream(getFile("zip64support.tar.bz2"))) {
+            new DumpArchiveInputStream(is).close();
             fail("expected an exception");
-        } catch (ArchiveException ex) {
+        } catch (final ArchiveException ex) {
             // expected
             assertTrue(ex.getCause() instanceof UnrecognizedFormatException);
-        } finally {
-            is.close();
         }
     }
 
+    @Test
     public void testConsumesArchiveCompletely() throws Exception {
-        InputStream is = DumpArchiveInputStreamTest.class
+        final InputStream is = DumpArchiveInputStreamTest.class
             .getResourceAsStream("/archive_with_trailer.dump");
-        DumpArchiveInputStream dump = new DumpArchiveInputStream(is);
+        final DumpArchiveInputStream dump = new DumpArchiveInputStream(is);
         while (dump.getNextDumpEntry() != null) {
             // just consume the archive
         }
-        byte[] expected = new byte[] {
+        final byte[] expected = new byte[] {
             'H', 'e', 'l', 'l', 'o', ',', ' ', 'w', 'o', 'r', 'l', 'd', '!', '\n'
         };
-        byte[] actual = new byte[expected.length];
+        final byte[] actual = new byte[expected.length];
         is.read(actual);
         assertArrayEquals(expected, actual);
         dump.close();
     }
 
+    @Test
+    public void singleByteReadConsistentlyReturnsMinusOneAtEof() throws Exception {
+        try (FileInputStream in = new FileInputStream(getFile("bla.dump"));
+             DumpArchiveInputStream archive = new DumpArchiveInputStream(in)) {
+            ArchiveEntry e = archive.getNextEntry();
+            IOUtils.toByteArray(archive);
+            assertEquals(-1, archive.read());
+            assertEquals(-1, archive.read());
+        }
+    }
+
+    @Test
+    public void multiByteReadConsistentlyReturnsMinusOneAtEof() throws Exception {
+        byte[] buf = new byte[2];
+        try (FileInputStream in = new FileInputStream(getFile("bla.dump"));
+             DumpArchiveInputStream archive = new DumpArchiveInputStream(in)) {
+            ArchiveEntry e = archive.getNextEntry();
+            IOUtils.toByteArray(archive);
+            assertEquals(-1, archive.read(buf));
+            assertEquals(-1, archive.read(buf));
+        }
+    }
+
 }
